@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
+# Copyright (c) 2016-2018 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -27,7 +27,7 @@
 
 __authors__ = ["V. Valls"]
 __license__ = "MIT"
-__date__ = "21/09/2017"
+__date__ = "24/04/2018"
 
 from silx.gui import qt
 from .DataViewer import DataViewer
@@ -113,6 +113,13 @@ class DataViewerFrame(qt.QWidget):
         """Called when the displayed view changes"""
         self.displayedViewChanged.emit(view)
 
+    def setGlobalHooks(self, hooks):
+        """Set a data view hooks for all the views
+
+        :param DataViewHooks context: The hooks to use
+        """
+        self.__dataViewer.setGlobalHooks(hooks)
+
     def availableViews(self):
         """Returns the list of registered views
 
@@ -133,7 +140,7 @@ class DataViewerFrame(qt.QWidget):
         overwriten to provide a different set of viewers.
 
         :param QWidget parent: QWidget parent of the views
-        :rtype: list[silx.gui.data.DataViews.DataView]
+        :rtype: List[silx.gui.data.DataViews.DataView]
         """
         return self.__dataViewer._createDefaultViews(parent)
 
@@ -192,3 +199,16 @@ class DataViewerFrame(qt.QWidget):
             - `ARRAY_MODE`: display the data as a table
         """
         return self.__dataViewer.setDisplayMode(modeId)
+
+    def getViewFromModeId(self, modeId):
+        """See :meth:`DataViewer.getViewFromModeId`"""
+        return self.__dataViewer.getViewFromModeId(modeId)
+
+    def replaceView(self, modeId, newView):
+        """Replace one of the builtin data views with a custom view.
+        See :meth:`DataViewer.replaceView` for more documentation.
+
+        :param DataViews.DataView newView: New data view
+        :return: True if replacement was successful, else False
+        """
+        return self.__dataViewer.replaceView(modeId, newView)
