@@ -27,12 +27,13 @@ data module to format data as text in the same way."""
 
 __authors__ = ["V. Valls"]
 __license__ = "MIT"
-__date__ = "27/09/2017"
+__date__ = "25/06/2018"
 
 import numpy
 import numbers
 from silx.third_party import six
 from silx.gui import qt
+import logging
 
 try:
     import h5py
@@ -40,6 +41,9 @@ except ImportError:
     h5py = None
 
 
+_logger = logging.getLogger(__name__)
+
+
 class TextFormatter(qt.QObject):
     """Formatter to convert data to string.
 
@@ -200,11 +204,12 @@ class TextFormatter(qt.QObject):
     def __formatBinary(self, data):
         if isinstance(data, numpy.void):
             if six.PY2:
-                data = [ord(d) for d in data.item()]
+                data = [ord(d) for d in data.data]
             else:
                 data = data.item().astype(numpy.uint8)
-        else:
+        elif six.PY2:
             data = [ord(d) for d in data]
+            # In python3 data is already a bytes array
         data = ["\\x%02X" % d for d in data]
         if self.__useQuoteForText:
             return "b\"%s\"" % "".join(data)
@@ -221,6 +226,30 @@ class TextFormatter(qt.QObject):
         else:
             return "".join(data)
 
+    def __formatCharString(self, data):
+        """Format text of char.
+
+        From the specifications we expect to have ASCII, but we also allow
+        CP1252 in some ceases as fallback.
+
+        If no encoding fits, it will display a readable ASCII chars, with
+        escaped chars (using the python syntax) for non decoded characters.
+
+        :param data: A binary string of char expected in ASCII
+        :rtype: str
+        """
+        try:
+            text = "%s" % data.decode("ascii")
+            return self.__formatText(text)
+        except UnicodeDecodeError:
+            # Here we can spam errors, this is definitly a badly
+            # generated file
+            _logger.error("Invalid ASCII string %s.", data)
+            if data == b"\xB0":
+                _logger.error("Fallback using cp1252 encoding")
+                return self.__formatText(u"\u00B0")
+        return self.__formatSafeAscii(data)
+
     def __formatH5pyObject(self, data, dtype):
         # That's an HDF5 object
         ref = h5py.check_dtype(ref=dtype)
@@ -236,11 +265,9 @@ class TextFormatter(qt.QObject):
                 return self.__formatText(data)
             elif vlen == six.binary_type:
                 # HDF5 ASCII
-                try:
-                    text = "%s" % data.decode("ascii")
-                    return self.__formatText(text)
-                except UnicodeDecodeError:
-                    return self.__formatSafeAscii(data)
+                return self.__formatCharString(data)
+            elif isinstance(vlen, numpy.dtype):
+                return self.toString(data, vlen)
         return None
 
     def toString(self, data, dtype=None):
@@ -266,24 +293,28 @@ class TextFormatter(qt.QObject):
             else:
                 text = [self.toString(d, dtype) for d in data]
                 return "[" + " ".join(text) + "]"
+        if dtype is not None and dtype.kind == 'O':
+            text = self.__formatH5pyObject(data, dtype)
+            if text is not None:
+                return text
         elif isinstance(data, numpy.void):
             if dtype is None:
                 dtype = data.dtype
-            if data.dtype.fields is not None:
-                text = [self.toString(data[f], dtype) for f in dtype.fields]
+            if dtype.fields is not None:
+                text = []
+                for index, field in enumerate(dtype.fields.items()):
+                    text.append(field[0] + ":" + self.toString(data[index], field[1][0]))
                 return "(" + " ".join(text) + ")"
             return self.__formatBinary(data)
         elif isinstance(data, (numpy.unicode_, six.text_type)):
             return self.__formatText(data)
         elif isinstance(data, (numpy.string_, six.binary_type)):
+            if dtype is None and hasattr(data, "dtype"):
+                dtype = data.dtype
             if dtype is not None:
                 # Maybe a sub item from HDF5
                 if dtype.kind == 'S':
-                    try:
-                        text = "%s" % data.decode("ascii")
-                        return self.__formatText(text)
-                    except UnicodeDecodeError:
-                        return self.__formatSafeAscii(data)
+                    return self.__formatCharString(data)
                 elif dtype.kind == 'O':
                     if h5py is not None:
                         text = self.__formatH5pyObject(data, dtype)
@@ -317,7 +348,7 @@ class TextFormatter(qt.QObject):
         elif isinstance(data, (numbers.Real, numpy.floating)):
             # It have to be done before complex checking
             return self.__floatFormat % data
-        elif isinstance(data, (numpy.complex_, numbers.Complex)):
+        elif isinstance(data, (numpy.complexfloating, numbers.Complex)):
             text = ""
             if data.real != 0:
                 text += self.__floatFormat % data.real
