@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
+# Copyright (c) 2016-2018 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -25,7 +25,7 @@
 
 __authors__ = ["V. Valls"]
 __license__ = "MIT"
-__date__ = "16/06/2017"
+__date__ = "25/06/2018"
 
 
 import logging
@@ -33,14 +33,9 @@ import re
 import numpy
 from .. import qt
 from .Hdf5TreeModel import Hdf5TreeModel
+import silx.io.utils
+from silx.gui import icons
 
-_logger = logging.getLogger(__name__)
-
-try:
-    import h5py
-except ImportError as e:
-    _logger.error("Module %s requires h5py", __name__)
-    raise e
 
 _logger = logging.getLogger(__name__)
 
@@ -51,6 +46,7 @@ class NexusSortFilterProxyModel(qt.QSortFilterProxyModel):
     def __init__(self, parent=None):
         qt.QSortFilterProxyModel.__init__(self, parent)
         self.__split = re.compile("(\\d+|\\D+)")
+        self.__iconCache = {}
 
     def lessThan(self, sourceLeft, sourceRight):
         """Returns True if the value of the item referred to by the given
@@ -86,12 +82,20 @@ class NexusSortFilterProxyModel(qt.QSortFilterProxyModel):
 
     def __isNXentry(self, node):
         """Returns true if the node is an NXentry"""
-        class_ = node.h5pyClass
-        if class_ is None or not issubclass(node.h5pyClass, h5py.Group):
+        class_ = node.h5Class
+        if class_ is None or class_ != silx.io.utils.H5Type.GROUP:
             return False
         nxClass = node.obj.attrs.get("NX_class", None)
         return nxClass == "NXentry"
 
+    def __isNXnode(self, node):
+        """Returns true if the node is an NX concept"""
+        class_ = node.h5Class
+        if class_ is None or class_ != silx.io.utils.H5Type.GROUP:
+            return False
+        nxClass = node.obj.attrs.get("NX_class", None)
+        return nxClass is not None
+
     def getWordsAndNumbers(self, name):
         """
         Returns a list of words and integers composing the name.
@@ -100,13 +104,16 @@ class NexusSortFilterProxyModel(qt.QSortFilterProxyModel):
         `["aaa", 10, "bbb", 50, ".", 30]`.
 
         :param str name: A name
-        :rtype: list
+        :rtype: List
         """
+        nonSensitive = self.sortCaseSensitivity() == qt.Qt.CaseInsensitive
         words = self.__split.findall(name)
         result = []
         for i in words:
             if i[0].isdigit():
                 i = int(i)
+            elif nonSensitive:
+                i = i.lower()
             result.append(i)
         return result
 
@@ -148,6 +155,50 @@ class NexusSortFilterProxyModel(qt.QSortFilterProxyModel):
             return left_time < right_time
         except KeyboardInterrupt:
             raise
-        except Exception as e:
+        except Exception:
             _logger.debug("Exception occurred", exc_info=True)
         return None
+
+    def __createCompoundIcon(self, backgroundIcon, foregroundIcon):
+        icon = qt.QIcon()
+
+        sizes = backgroundIcon.availableSizes()
+        sizes = sorted(sizes, key=lambda s: s.height())
+        sizes = filter(lambda s: s.height() < 100, sizes)
+        sizes = list(sizes)
+        if len(sizes) > 0:
+            baseSize = sizes[-1]
+        else:
+            baseSize = qt.QSize(32, 32)
+
+        modes = [qt.QIcon.Normal, qt.QIcon.Disabled]
+        for mode in modes:
+            pixmap = qt.QPixmap(baseSize)
+            pixmap.fill(qt.Qt.transparent)
+            painter = qt.QPainter(pixmap)
+            painter.drawPixmap(0, 0, backgroundIcon.pixmap(baseSize, mode=mode))
+            painter.drawPixmap(0, 0, foregroundIcon.pixmap(baseSize, mode=mode))
+            painter.end()
+            icon.addPixmap(pixmap, mode=mode)
+
+        return icon
+
+    def __getNxIcon(self, baseIcon):
+        iconHash = baseIcon.cacheKey()
+        icon = self.__iconCache.get(iconHash, None)
+        if icon is None:
+            nxIcon = icons.getQIcon("layer-nx")
+            icon = self.__createCompoundIcon(baseIcon, nxIcon)
+            self.__iconCache[iconHash] = icon
+        return icon
+
+    def data(self, index, role=qt.Qt.DisplayRole):
+        result = super(NexusSortFilterProxyModel, self).data(index, role)
+
+        if index.column() == Hdf5TreeModel.NAME_COLUMN:
+            if role == qt.Qt.DecorationRole:
+                sourceIndex = self.mapToSource(index)
+                item = self.sourceModel().data(sourceIndex, Hdf5TreeModel.H5PY_ITEM_ROLE)
+                if self.__isNXnode(item):
+                    result = self.__getNxIcon(result)
+        return result
