@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2004-2017 European Synchrotron Radiation Facility
+# Copyright (c) 2004-2018 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -28,7 +28,7 @@ and stacks of images"""
 
 __authors__ = ["V.A. Sole", "T. Vincent", "P. Knobel", "H. Payno"]
 __license__ = "MIT"
-__date__ = "17/08/2017"
+__date__ = "24/04/2018"
 
 
 import weakref
@@ -40,7 +40,7 @@ from silx.image.bilinear import BilinearImage
 from .. import icons
 from .. import qt
 from . import items
-from .Colors import cursorColorForColormap
+from ..colors import cursorColorForColormap
 from . import actions
 from .PlotToolButtons import ProfileToolButton
 from .ProfileMainWindow import ProfileMainWindow
@@ -637,6 +637,12 @@ class ProfileToolBar(qt.QToolBar):
                                            colormap=colormap)
         else:
             coords = numpy.arange(len(profile[0]), dtype=numpy.float32)
+            # Scale horizontal and vertical profile coordinates
+            if self._roiInfo[2] == 'X':
+                coords = coords * scale[0] + origin[0]
+            elif self._roiInfo[2] == 'Y':
+                coords = coords * scale[1] + origin[1]
+
             self.getProfilePlot().addCurve(coords,
                                            profile[0],
                                            legend=profileName,
@@ -660,23 +666,23 @@ class ProfileToolBar(qt.QToolBar):
             winGeom = self.window().frameGeometry()
             qapp = qt.QApplication.instance()
             screenGeom = qapp.desktop().availableGeometry(self)
-
             spaceOnLeftSide = winGeom.left()
             spaceOnRightSide = screenGeom.width() - winGeom.right()
 
             profileWindowWidth = profileMainWindow.frameGeometry().width()
-            if (profileWindowWidth < spaceOnRightSide or
-                    spaceOnRightSide > spaceOnLeftSide):
+            if (profileWindowWidth < spaceOnRightSide):
                 # Place profile on the right
                 profileMainWindow.move(winGeom.right(), winGeom.top())
-            else:
-                # Not enough place on the right, place profile on the left
+            elif(profileWindowWidth < spaceOnLeftSide):
+                # Place profile on the left
                 profileMainWindow.move(
-                        max(0, winGeom.left() - profileWindowWidth), winGeom.top())
+                    max(0, winGeom.left() - profileWindowWidth), winGeom.top())
 
             profileMainWindow.show()
+            profileMainWindow.raise_()
         else:
             self.getProfilePlot().show()
+            self.getProfilePlot().raise_()
 
     def hideProfileWindow(self):
         """Hide profile window.
