@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
+# Copyright (c) 2016-2018 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -69,19 +69,22 @@ Example::
 
 __authors__ = ["P. Knobel", "H. Payno"]
 __license__ = "MIT"
-__date__ = "11/09/2017"
+__date__ = "26/04/2018"
 
 import numpy
+import logging
 
+import silx
 from silx.gui import qt
 from .. import icons
 from . import items, PlotWindow, actions
-from .Colormap import Colormap
-from .Colors import cursorColorForColormap
-from .PlotTools import LimitsToolBar
+from ..colors import Colormap
+from ..colors import cursorColorForColormap
+from .tools import LimitsToolBar
 from .Profile import Profile3DToolBar
 from ..widgets.FrameBrowser import HorizontalSliderWithBrowser
 
+from silx.gui.plot.actions import control as actions_control
 from silx.utils.array_like import DatasetView, ListOfImages
 from silx.math import calibration
 from silx.utils.deprecation import deprecated_warning
@@ -95,6 +98,8 @@ except ImportError:
 else:
     from silx.io.utils import is_dataset
 
+_logger = logging.getLogger(__name__)
+
 
 class StackView(qt.QMainWindow):
     """Stack view widget, to display and browse through stack of
@@ -155,6 +160,12 @@ class StackView(qt.QMainWindow):
     integer.
     """
 
+    sigFrameChanged = qt.Signal(int)
+    """Signal emitter when the frame number has changed.
+
+    This signal provides the current frame number.
+    """
+
     def __init__(self, parent=None, resetzoom=True, backend=None,
                  autoScale=False, logScale=False, grid=False,
                  colormap=True, aspectRatio=True, yinverted=True,
@@ -205,6 +216,9 @@ class StackView(qt.QMainWindow):
         self.sigActiveImageChanged = self._plot.sigActiveImageChanged
         self.sigPlotSignal = self._plot.sigPlotSignal
 
+        if silx.config.DEFAULT_PLOT_IMAGE_Y_AXIS_ORIENTATION == 'downward':
+            self._plot.getYAxis().setInverted(True)
+
         self._addColorBarAction()
 
         self._plot.profile = Profile3DToolBar(parent=self._plot,
@@ -220,6 +234,7 @@ class StackView(qt.QMainWindow):
         self._browser_label = qt.QLabel("Image index (Dim0):")
 
         self._browser = HorizontalSliderWithBrowser(central_widget)
+        self._browser.setRange(0, 0)
         self._browser.valueChanged[int].connect(self.__updateFrameNumber)
         self._browser.setEnabled(False)
 
@@ -245,9 +260,8 @@ class StackView(qt.QMainWindow):
         for index, action in enumerate(actions):
             if action is self._plot.getColormapAction():
                 break
-        self._plot.toolBar().insertAction(
-            actions[index + 1],
-            self._plot.getColorBarWidget().getToggleViewAction())
+        self._colorbarAction = actions_control.ColorBarAction(self._plot, self._plot)
+        self._plot.toolBar().insertAction(actions[index + 1], self._colorbarAction)
 
     def _plotCallback(self, eventDict):
         """Callback for plot events.
@@ -313,7 +327,7 @@ class StackView(qt.QMainWindow):
         assert self._stack is not None
         assert 0 <= self._perspective < 3
 
-        # ensure we have the stack encapsulated in an array like object
+        # ensure we have the stack encapsulated in an array-like object
         # having a transpose() method
         if isinstance(self._stack, numpy.ndarray):
             self.__transposed_view = self._stack
@@ -324,7 +338,7 @@ class StackView(qt.QMainWindow):
         elif isinstance(self._stack, ListOfImages):
             self.__transposed_view = ListOfImages(self._stack)
 
-        # transpose the array like object if necessary
+        # transpose the array-like object if necessary
         if self._perspective == 1:
             self.__transposed_view = self.__transposed_view.transpose((1, 0, 2))
         elif self._perspective == 2:
@@ -338,13 +352,16 @@ class StackView(qt.QMainWindow):
 
         :param index: index of the frame to be displayed
         """
-        assert self.__transposed_view is not None
+        if self.__transposed_view is None:
+            # no data set
+            return
         self._plot.addImage(self.__transposed_view[index, :, :],
                             origin=self._getImageOrigin(),
                             scale=self._getImageScale(),
                             legend=self.__imageLegend,
-                            resetzoom=False, replace=False)
+                            resetzoom=False)
         self._updateTitle()
+        self.sigFrameChanged.emit(index)
 
     def _set3DScaleAndOrigin(self, calibrations):
         """Set scale and origin for all 3 axes, to be used when plotting
@@ -358,7 +375,7 @@ class StackView(qt.QMainWindow):
                                    calibration.NoCalibration())
         else:
             self.calibrations3D = []
-            for calib in calibrations:
+            for i, calib in enumerate(calibrations):
                 if hasattr(calib, "__len__") and len(calib) == 2:
                     calib = calibration.LinearCalibration(calib[0], calib[1])
                 elif calib is None:
@@ -367,9 +384,19 @@ class StackView(qt.QMainWindow):
                     raise TypeError("calibration must be a 2-tuple, None or" +
                                     " an instance of an AbstractCalibration " +
                                     "subclass")
+                elif not calib.is_affine():
+                    _logger.warning(
+                            "Calibration for dimension %d is not linear, "
+                            "it will be ignored for scaling the graph axes.",
+                            i)
                 self.calibrations3D.append(calib)
 
     def _getXYZCalibs(self):
+        """Return calibrations sorted in the XYZ graph order.
+
+        If the X or Y calibration is not linear, it will be replaced
+        with a :class:`calibration.NoCalibration` object
+        and as a result the corresponding axis will not be scaled."""
         xy_dims = [0, 1, 2]
         xy_dims.remove(self._perspective)
 
@@ -377,6 +404,12 @@ class StackView(qt.QMainWindow):
         ycalib = self.calibrations3D[min(xy_dims)]
         zcalib = self.calibrations3D[self._perspective]
 
+        # filter out non-linear calibration for graph axes
+        if not xcalib.is_affine():
+            xcalib = calibration.NoCalibration()
+        if not ycalib.is_affine():
+            ycalib = calibration.NoCalibration()
+
         return xcalib, ycalib, zcalib
 
     def _getImageScale(self):
@@ -469,6 +502,7 @@ class StackView(qt.QMainWindow):
                             colormap=self.getColormap(),
                             origin=self._getImageOrigin(),
                             scale=self._getImageScale(),
+                            replace=True,
                             resetzoom=False)
         self._plot.setActiveImage(self.__imageLegend)
         self._plot.setGraphTitle("Image z=%g" % self._getImageZ(0))
@@ -586,6 +620,14 @@ class StackView(qt.QMainWindow):
         """
         self._browser.setValue(number)
 
+    def getFrameNumber(self):
+        """Set the frame selection to a specific value
+
+        :return: Index of currently displayed frame
+        :rtype: int
+        """
+        return self._browser.value()
+
     def setFirstStackDimension(self, first_stack_dimension):
         """When viewing the last 3 dimensions of an n-D array (n>3), you can
         use this method to change the text in the combobox.
@@ -641,6 +683,8 @@ class StackView(qt.QMainWindow):
         self.__transposed_view = None
         self._perspective = 0
         self._browser.setEnabled(False)
+        # reset browser range
+        self._browser.setRange(0, 0)
         self._plot.clear()
 
     def setLabels(self, labels=None):
@@ -652,7 +696,7 @@ class StackView(qt.QMainWindow):
         when the volume is rotated (when different axes are selected as the
         X and Y axes).
 
-        :param list(str) labels: 3 labels corresponding to the 3 dimensions
+        :param List[str] labels: 3 labels corresponding to the 3 dimensions
              of the data volumes.
         """
 
@@ -972,6 +1016,16 @@ class StackView(qt.QMainWindow):
         """
         return self._plot.getActiveImage(just_legend=just_legend)
 
+    def getColorBarAction(self):
+        """Returns the action managing the visibility of the colorbar.
+
+        .. warning:: to show/hide the plot colorbar call directly the ColorBar
+            widget using getColorBarWidget()
+
+        :rtype: QAction
+        """
+        return self._colorbarAction
+
     def remove(self, legend=None,
                kind=('curve', 'image', 'item', 'marker')):
         """See :meth:`Plot.Plot.remove`"""
@@ -1091,18 +1145,18 @@ class StackViewMainWindow(StackView):
         self.statusBar()
 
         menu = self.menuBar().addMenu('File')
-        menu.addAction(self._plot.saveAction)
-        menu.addAction(self._plot.printAction)
+        menu.addAction(self._plot.getOutputToolBar().getSaveAction())
+        menu.addAction(self._plot.getOutputToolBar().getPrintAction())
         menu.addSeparator()
         action = menu.addAction('Quit')
         action.triggered[bool].connect(qt.QApplication.instance().quit)
 
         menu = self.menuBar().addMenu('Edit')
-        menu.addAction(self._plot.copyAction)
+        menu.addAction(self._plot.getOutputToolBar().getCopyAction())
         menu.addSeparator()
-        menu.addAction(self._plot.resetZoomAction)
-        menu.addAction(self._plot.colormapAction)
-        menu.addAction(self._plot.getColorBarWidget().getToggleViewAction())
+        menu.addAction(self._plot.getResetZoomAction())
+        menu.addAction(self._plot.getColormapAction())
+        menu.addAction(self.getColorBarAction())
 
         menu.addAction(actions.control.KeepAspectRatioAction(self._plot, self))
         menu.addAction(actions.control.YAxisInvertedAction(self._plot, self))
