@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2017 European Synchrotron Radiation Facility
+# Copyright (c) 2017-2018 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -27,18 +27,19 @@
 
 __authors__ = ["T. Vincent"]
 __license__ = "MIT"
-__date__ = "27/06/2017"
+__date__ = "14/06/2018"
 
 import collections
 from copy import deepcopy
 import logging
+import warnings
 import weakref
 import numpy
 from silx.third_party import six, enum
 
 from ... import qt
-from .. import Colors
-from ..Colormap import Colormap
+from ... import colors
+from ...colors import Colormap
 
 
 _logger = logging.getLogger(__name__)
@@ -115,6 +116,9 @@ class ItemChangedType(enum.Enum):
     OVERLAY = 'overlayChanged'
     """Item's overlay state changed flag."""
 
+    VISUALIZATION_MODE = 'visualizationModeChanged'
+    """Item's visualization mode changed flag."""
+
 
 class Item(qt.QObject):
     """Description of an item of the plot"""
@@ -136,7 +140,7 @@ class Item(qt.QObject):
     """
 
     def __init__(self):
-        super(Item, self).__init__()
+        qt.QObject.__init__(self)
         self._dirty = True
         self._plotRef = None
         self._visible = True
@@ -312,7 +316,24 @@ class Item(qt.QObject):
 
 # Mix-in classes ##############################################################
 
-class LabelsMixIn(object):
+class ItemMixInBase(qt.QObject):
+    """Base class for Item mix-in"""
+
+    def _updated(self, event=None, checkVisibility=True):
+        """This is implemented in :class:`Item`.
+
+        Mark the item as dirty (i.e., needing update).
+        This also triggers Plot.replot.
+
+        :param event: The event to send to :attr:`sigItemChanged` signal.
+        :param bool checkVisibility: True to only mark as dirty if visible,
+                                     False to always mark as dirty.
+        """
+        raise RuntimeError(
+            "Issue with Mix-In class inheritance order")
+
+
+class LabelsMixIn(ItemMixInBase):
     """Mix-in class for items with x and y labels
 
     Setters are private, otherwise it needs to check the plot
@@ -352,7 +373,7 @@ class LabelsMixIn(object):
         self._ylabel = str(label)
 
 
-class DraggableMixIn(object):
+class DraggableMixIn(ItemMixInBase):
     """Mix-in class for draggable items"""
 
     def __init__(self):
@@ -375,7 +396,7 @@ class DraggableMixIn(object):
         self._draggable = bool(draggable)
 
 
-class ColormapMixIn(object):
+class ColormapMixIn(ItemMixInBase):
     """Mix-in class for items with colormap"""
 
     def __init__(self):
@@ -389,7 +410,7 @@ class ColormapMixIn(object):
     def setColormap(self, colormap):
         """Set the colormap of this image
 
-        :param Colormap colormap: colormap description
+        :param silx.gui.colors.Colormap colormap: colormap description
         """
         if isinstance(colormap, dict):
             colormap = Colormap._fromDict(colormap)
@@ -406,7 +427,7 @@ class ColormapMixIn(object):
         self._updated(ItemChangedType.COLORMAP)
 
 
-class SymbolMixIn(object):
+class SymbolMixIn(ItemMixInBase):
     """Mix-in class for items with symbol type"""
 
     _DEFAULT_SYMBOL = ''
@@ -415,10 +436,49 @@ class SymbolMixIn(object):
     _DEFAULT_SYMBOL_SIZE = 6.0
     """Default marker size of the item"""
 
+    _SUPPORTED_SYMBOLS = collections.OrderedDict((
+        ('o', 'Circle'),
+        ('d', 'Diamond'),
+        ('s', 'Square'),
+        ('+', 'Plus'),
+        ('x', 'Cross'),
+        ('.', 'Point'),
+        (',', 'Pixel'),
+        ('', 'None')))
+    """Dict of supported symbols"""
+
     def __init__(self):
         self._symbol = self._DEFAULT_SYMBOL
         self._symbol_size = self._DEFAULT_SYMBOL_SIZE
 
+    @classmethod
+    def getSupportedSymbols(cls):
+        """Returns the list of supported symbol names.
+
+        :rtype: tuple of str
+        """
+        return tuple(cls._SUPPORTED_SYMBOLS.keys())
+
+    @classmethod
+    def getSupportedSymbolNames(cls):
+        """Returns the list of supported symbol human-readable names.
+
+        :rtype: tuple of str
+        """
+        return tuple(cls._SUPPORTED_SYMBOLS.values())
+
+    def getSymbolName(self, symbol=None):
+        """Returns human-readable name for a symbol.
+
+        :param str symbol: The symbol from which to get the name.
+                           Default: current symbol.
+        :rtype: str
+        :raise KeyError: if symbol is not in :meth:`getSupportedSymbols`.
+        """
+        if symbol is None:
+            symbol = self.getSymbol()
+        return self._SUPPORTED_SYMBOLS[symbol]
+
     def getSymbol(self):
         """Return the point marker type.
 
@@ -441,11 +501,19 @@ class SymbolMixIn(object):
 
         See :meth:`getSymbol`.
 
-        :param str symbol: Marker type
+        :param str symbol: Marker type or marker name
         """
-        assert symbol in ('o', '.', ',', '+', 'x', 'd', 's', '', None)
         if symbol is None:
             symbol = self._DEFAULT_SYMBOL
+
+        elif symbol not in self.getSupportedSymbols():
+            for symbolCode, name in self._SUPPORTED_SYMBOLS.items():
+                if name.lower() == symbol.lower():
+                    symbol = symbolCode
+                    break
+            else:
+                raise ValueError('Unsupported symbol %s' % str(symbol))
+
         if symbol != self._symbol:
             self._symbol = symbol
             self._updated(ItemChangedType.SYMBOL)
@@ -471,7 +539,7 @@ class SymbolMixIn(object):
             self._updated(ItemChangedType.SYMBOL_SIZE)
 
 
-class LineMixIn(object):
+class LineMixIn(ItemMixInBase):
     """Mix-in class for item with line"""
 
     _DEFAULT_LINEWIDTH = 1.
@@ -531,7 +599,7 @@ class LineMixIn(object):
             self._updated(ItemChangedType.LINE_STYLE)
 
 
-class ColorMixIn(object):
+class ColorMixIn(ItemMixInBase):
     """Mix-in class for item with color"""
 
     _DEFAULT_COLOR = (0., 0., 0., 1.)
@@ -552,17 +620,17 @@ class ColorMixIn(object):
 
         :param color: color(s) to be used
         :type color: str ("#RRGGBB") or (npoints, 4) unsigned byte array or
-                     one of the predefined color names defined in Colors.py
+                     one of the predefined color names defined in colors.py
         :param bool copy: True (Default) to get a copy,
                          False to use internal representation (do not modify!)
         """
         if isinstance(color, six.string_types):
-            color = Colors.rgba(color)
+            color = colors.rgba(color)
         else:
             color = numpy.array(color, copy=copy)
             # TODO more checks + improve color array support
             if color.ndim == 1:  # Single RGBA color
-                color = Colors.rgba(color)
+                color = colors.rgba(color)
             else:  # Array of colors
                 assert color.ndim == 2
 
@@ -570,7 +638,7 @@ class ColorMixIn(object):
         self._updated(ItemChangedType.COLOR)
 
 
-class YAxisMixIn(object):
+class YAxisMixIn(ItemMixInBase):
     """Mix-in class for item with yaxis"""
 
     _DEFAULT_YAXIS = 'left'
@@ -600,7 +668,7 @@ class YAxisMixIn(object):
             self._updated(ItemChangedType.YAXIS)
 
 
-class FillMixIn(object):
+class FillMixIn(ItemMixInBase):
     """Mix-in class for item with fill"""
 
     def __init__(self):
@@ -624,7 +692,7 @@ class FillMixIn(object):
             self._updated(ItemChangedType.FILL)
 
 
-class AlphaMixIn(object):
+class AlphaMixIn(ItemMixInBase):
     """Mix-in class for item with opacity"""
 
     def __init__(self):
@@ -700,7 +768,10 @@ class Points(Item, SymbolMixIn, AlphaMixIn):
                 error = numpy.ravel(error)
 
             # Supports error being scalar, N or 2xN array
-            errorClipped = (value - numpy.atleast_2d(error)[0]) <= 0
+            valueMinusError = value - numpy.atleast_2d(error)[0]
+            errorClipped = numpy.isnan(valueMinusError)
+            mask = numpy.logical_not(errorClipped)
+            errorClipped[mask] = valueMinusError[mask] <= 0
 
             if numpy.any(errorClipped):  # Need filtering
 
@@ -738,10 +809,20 @@ class Points(Item, SymbolMixIn, AlphaMixIn):
         """
         assert xPositive or yPositive
         if (xPositive, yPositive) not in self._clippedCache:
-            x = self.getXData(copy=False)
-            y = self.getYData(copy=False)
-            xclipped = (x <= 0) if xPositive else False
-            yclipped = (y <= 0) if yPositive else False
+            xclipped, yclipped = False, False
+
+            if xPositive:
+                x = self.getXData(copy=False)
+                with warnings.catch_warnings():  # Ignore NaN warnings
+                    warnings.simplefilter('ignore', category=RuntimeWarning)
+                    xclipped = x <= 0
+
+            if yPositive:
+                y = self.getYData(copy=False)
+                with warnings.catch_warnings():  # Ignore NaN warnings
+                    warnings.simplefilter('ignore', category=RuntimeWarning)
+                    yclipped = y <= 0
+
             self._clippedCache[(xPositive, yPositive)] = \
                 numpy.logical_or(xclipped, yclipped)
         return self._clippedCache[(xPositive, yPositive)]
