@@ -26,7 +26,7 @@
 
 __authors__ = ["T. Vincent"]
 __license__ = "MIT"
-__date__ = "01/09/2017"
+__date__ = "17/01/2018"
 
 
 import logging
@@ -36,7 +36,8 @@ import unittest
 import numpy
 
 from silx.gui import qt
-from silx.test.utils import temp_dir, ParametricTestCase
+from silx.test.utils import temp_dir
+from silx.utils.testutils import ParametricTestCase
 from silx.gui.test.utils import getQToolButtonFromAction
 from silx.gui.plot import PlotWindow, MaskToolsWidget
 from .utils import PlotWidgetTestCase
@@ -84,8 +85,10 @@ class TestMaskToolsWidget(PlotWidgetTestCase, ParametricTestCase):
         pos0 = xCenter, yCenter
         pos1 = xCenter + offset, yCenter + offset
 
+        self.mouseMove(plot, pos=(0, 0))
         self.mouseMove(plot, pos=pos0)
         self.mousePress(plot, qt.Qt.LeftButton, pos=pos0)
+        self.mouseMove(plot, pos=(0, 0))
         self.mouseMove(plot, pos=pos1)
         self.mouseRelease(plot, qt.Qt.LeftButton, pos=pos1)
 
@@ -194,7 +197,7 @@ class TestMaskToolsWidget(PlotWidgetTestCase, ParametricTestCase):
                 self.assertIsNot(toolButton, None)
                 self.mouseClick(toolButton, qt.Qt.LeftButton)
 
-                self.maskWidget.pencilSpinBox.setValue(10)
+                self.maskWidget.pencilSpinBox.setValue(30)
                 self.qapp.processEvents()
 
                 # mask
