@@ -27,7 +27,7 @@ from __future__ import absolute_import, division, unicode_literals
 
 __authors__ = ["T. Vincent"]
 __license__ = "MIT"
-__date__ = "25/07/2016"
+__date__ = "24/04/2018"
 
 
 import collections
@@ -38,7 +38,7 @@ import string
 
 import numpy
 
-from silx.gui.plot.Colors import rgba
+from silx.gui.colors import rgba
 
 from ... import _glutils
 from ..._glutils import gl
@@ -1246,12 +1246,12 @@ class _Points(Geometry):
         $clippingCall(vCameraPosition);
 
         float alpha = alphaSymbol(gl_PointCoord, vSize);
-        if (alpha == 0.0) {
-            discard;
-        }
 
         gl_FragColor = $valueToColorCall(vValue);
         gl_FragColor.a *= alpha;
+        if (gl_FragColor.a == 0.0) {
+            discard;
+        }
     }
     """))
 
