@@ -32,7 +32,9 @@ from numpy.distutils.misc_util import Configuration
 
 def configuration(parent_package='', top_path=None):
     config = Configuration('plot3d', parent_package, top_path)
+    config.add_subpackage('_model')
     config.add_subpackage('actions')
+    config.add_subpackage('items')
     config.add_subpackage('scene')
     config.add_subpackage('tools')
     config.add_subpackage('test')
