@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2004-2017 European Synchrotron Radiation Facility
+# Copyright (c) 2004-2018 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -31,7 +31,7 @@ The user can interactively move and resize the items.
 """
 import sys
 import logging
-from silx.gui import qt
+from silx.gui import qt, printer
 
 
 __authors__ = ["V.A. Sole", "P. Knobel"]
@@ -387,7 +387,7 @@ class PrintPreviewDialog(qt.QDialog):
         *None*.
         """
         if self.printer is None:
-            self.printer = qt.QPrinter()
+            self.printer = printer.getDefaultPrinter()
         if self.printDialog is None:
             self.printDialog = qt.QPrintDialog(self.printer, self)
         if self.printDialog.exec_():
@@ -666,8 +666,6 @@ class _GraphicsResizeRectItem(qt.QGraphicsRectItem):
 def main():
     """
     """
-    import sys
-
     if len(sys.argv) < 2:
         print("give an image file as parameter please.")
         sys.exit(1)
