@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
+# Copyright (c) 2016-2018 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -29,6 +29,8 @@ from . import test_tablewidget
 from . import test_threadpoolpushbutton
 from . import test_hierarchicaltableview
 from . import test_printpreview
+from . import test_framebrowser
+from . import test_boxlayoutdockwidget
 
 __authors__ = ["V. Valls", "P. Knobel"]
 __license__ = "MIT"
@@ -43,5 +45,7 @@ def suite():
          test_periodictable.suite(),
          test_printpreview.suite(),
          test_hierarchicaltableview.suite(),
+         test_framebrowser.suite(),
+         test_boxlayoutdockwidget.suite(),
          ])
     return test_suite
