@@ -1,6 +1,6 @@
 # coding: utf-8
 # /*##########################################################################
-# Copyright (C) 2016-2017 European Synchrotron Radiation Facility
+# Copyright (C) 2016-2018 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -32,12 +32,13 @@ import collections
 import h5py
 import numpy
 from silx.third_party import six
+import weakref
 
-from .utils import is_dataset
+from . import utils
 
 __authors__ = ["V. Valls", "P. Knobel"]
 __license__ = "MIT"
-__date__ = "02/10/2017"
+__date__ = "02/07/2018"
 
 
 class _MappingProxyType(collections.MutableMapping):
@@ -90,7 +91,7 @@ class Node(object):
     """
 
     def __init__(self, name, parent=None, attrs=None):
-        self.__parent = parent
+        self._set_parent(parent)
         self.__basename = name
         self.__attrs = {}
         if attrs is not None:
@@ -99,13 +100,32 @@ class Node(object):
     def _set_basename(self, name):
         self.__basename = name
 
+    @property
+    def h5_class(self):
+        """Returns the HDF5 class which is mimicked by this class.
+
+        :rtype: H5Type
+        """
+        raise NotImplementedError()
+
     @property
     def h5py_class(self):
         """Returns the h5py classes which is mimicked by this class. It can be
         one of `h5py.File, h5py.Group` or `h5py.Dataset`
 
+        This should not be used anymore. Prefer using `h5_class`
+
         :rtype: Class
         """
+        h5_class = self.h5_class
+        if h5_class == utils.H5Type.FILE:
+            return h5py.File
+        elif h5_class == utils.H5Type.GROUP:
+            return h5py.Group
+        elif h5_class == utils.H5Type.DATASET:
+            return h5py.Dataset
+        elif h5_class == utils.H5Type.SOFT_LINK:
+            return h5py.SoftLink
         raise NotImplementedError()
 
     @property
@@ -114,21 +134,13 @@ class Node(object):
 
         :rtype: Node
         """
-        return self.__parent
-
-    @property
-    def file(self):
-        """Returns the file node of this node.
-
-        :rtype: Node
-        """
-        node = self
-        while node.__parent is not None:
-            node = node.__parent
-        if isinstance(node, File):
-            return node
+        if self.__parent is None:
+            parent = None
         else:
-            return None
+            parent = self.__parent()
+            if parent is None:
+                self.__parent = None
+        return parent
 
     def _set_parent(self, parent):
         """Set the parent of this node.
@@ -137,7 +149,24 @@ class Node(object):
 
         :param Node parent: New parent for this node
         """
-        self.__parent = parent
+        if parent is not None:
+            self.__parent = weakref.ref(parent)
+        else:
+            self.__parent = None
+
+    @property
+    def file(self):
+        """Returns the file node of this node.
+
+        :rtype: Node
+        """
+        node = self
+        while node.parent is not None:
+            node = node.parent
+        if isinstance(node, File):
+            return node
+        else:
+            return None
 
     @property
     def attrs(self):
@@ -154,11 +183,12 @@ class Node(object):
     def name(self):
         """Returns the HDF5 name of this node.
         """
-        if self.__parent is None:
+        parent = self.parent
+        if parent is None:
             return "/"
-        if self.__parent.name == "/":
+        if parent.name == "/":
             return "/" + self.basename
-        return self.__parent.name + "/" + self.basename
+        return parent.name + "/" + self.basename
 
     @property
     def basename(self):
@@ -233,13 +263,12 @@ class Dataset(Node):
         return self.__data
 
     @property
-    def h5py_class(self):
-        """Returns the h5py classes which is mimicked by this class. It can be
-        one of `h5py.File, h5py.Group` or `h5py.Dataset`
+    def h5_class(self):
+        """Returns the HDF5 class which is mimicked by this class.
 
-        :rtype: Class
+        :rtype: H5Type
         """
-        return h5py.Dataset
+        return utils.H5Type.DATASET
 
     @property
     def dtype(self):
@@ -358,7 +387,7 @@ class Dataset(Node):
     # make comparisons and operations on the data
     def __eq__(self, other):
         """When comparing datasets, compare the actual data."""
-        if is_dataset(other):
+        if utils.is_dataset(other):
             return self[()] == other[()]
         return self[()] == other
 
@@ -414,31 +443,31 @@ class Dataset(Node):
         return self.__bool__()
 
     def __ne__(self, other):
-        if is_dataset(other):
+        if utils.is_dataset(other):
             return self[()] != other[()]
         else:
             return self[()] != other
 
     def __lt__(self, other):
-        if is_dataset(other):
+        if utils.is_dataset(other):
             return self[()] < other[()]
         else:
             return self[()] < other
 
     def __le__(self, other):
-        if is_dataset(other):
+        if utils.is_dataset(other):
             return self[()] <= other[()]
         else:
             return self[()] <= other
 
     def __gt__(self, other):
-        if is_dataset(other):
+        if utils.is_dataset(other):
             return self[()] > other[()]
         else:
             return self[()] > other
 
     def __ge__(self, other):
-        if is_dataset(other):
+        if utils.is_dataset(other):
             return self[()] >= other[()]
         else:
             return self[()] >= other
@@ -453,6 +482,35 @@ class Dataset(Node):
         raise AttributeError("Dataset has no attribute %s" % item)
 
 
+class DatasetProxy(Dataset):
+    """Virtual dataset providing content of another dataset"""
+
+    def __init__(self, name, target, parent=None):
+        Dataset.__init__(self, name, data=None, parent=parent)
+        if not utils.is_dataset(target):
+            raise TypeError("A Dataset is expected but %s found", target.__class__)
+        self.__target = target
+
+    @property
+    def shape(self):
+        return self.__target.shape
+
+    @property
+    def size(self):
+        return self.__target.size
+
+    @property
+    def dtype(self):
+        return self.__target.dtype
+
+    def _get_data(self):
+        return self.__target[...]
+
+    @property
+    def attrs(self):
+        return self.__target.attrs
+
+
 class _LinkToDataset(Dataset):
     """Virtual dataset providing link to another dataset"""
 
@@ -521,13 +579,12 @@ class SoftLink(Node):
         self.target = str(path)
 
     @property
-    def h5py_class(self):
-        """Returns the h5py class which is mimicked by this class
-        (:class:`h5py.SoftLink`).
+    def h5_class(self):
+        """Returns the HDF5 class which is mimicked by this class.
 
-        :rtype: Class
+        :rtype: H5Type
         """
-        return h5py.SoftLink
+        return utils.H5Type.SOFT_LINK
 
     @property
     def path(self):
@@ -558,14 +615,12 @@ class Group(Node):
         node._set_parent(self)
 
     @property
-    def h5py_class(self):
-        """Returns the h5py classes which is mimicked by this class.
+    def h5_class(self):
+        """Returns the HDF5 class which is mimicked by this class.
 
-        It returns `h5py.Group`
-
-        :rtype: Class
+        :rtype: H5Type
         """
-        return h5py.Group
+        return utils.H5Type.GROUP
 
     def _get(self, name, getlink):
         """If getlink is True and name points to an existing SoftLink, this
@@ -643,9 +698,8 @@ class Group(Node):
             node = h5py.HardLink()
 
         if getclass:
-            if hasattr(node, "h5py_class"):
-                obj = node.h5py_class
-            else:
+            obj = utils.get_h5py_class(node)
+            if obj is None:
                 obj = node.__class__
         else:
             obj = node
@@ -709,7 +763,7 @@ class Group(Node):
     def __getitem__(self, name):
         """Return a child from his name.
 
-        :param name str: name of a member or a path throug members using '/'
+        :param str name: name of a member or a path throug members using '/'
             separator. A '/' as a prefix access to the root item of the tree.
         :rtype: Node
         """
@@ -743,7 +797,7 @@ class Group(Node):
                     if node is None:
                         # broken link
                         return False
-                if node.h5py_class == h5py.Dataset:
+                if utils.is_dataset(node):
                     return False
                 continue
             if basename not in node._get_items():
@@ -806,7 +860,7 @@ class Group(Node):
         See the documentation for `h5py.Group.visit` for more help.
 
         :param func: Callable (function, method or callable object)
-        :type func: function
+        :type func: callable
         """
         origin_name = self.name
         return self._visit(func, origin_name, visit_links)
@@ -816,7 +870,7 @@ class Group(Node):
         See the documentation for `h5py.Group.visititems` for more help.
 
         :param func: Callable (function, method or callable object)
-        :type func: function
+        :type func: callable
         :param bool visit_links: If *False*, ignore links. If *True*,
             call `func(name)` for links and recurse into target groups.
         """
@@ -989,9 +1043,9 @@ class File(Group):
         return self._mode
 
     @property
-    def h5py_class(self):
+    def h5_class(self):
         """Returns the :class:`h5py.File` class"""
-        return h5py.File
+        return utils.H5Type.FILE
 
     def __enter__(self):
         return self
