@@ -25,7 +25,7 @@
 
 __authors__ = ["V. Valls"]
 __license__ = "MIT"
-__date__ = "24/02/2018"
+__date__ = "02/07/2018"
 
 import os
 import logging
@@ -226,7 +226,7 @@ class TestFabioH5(unittest.TestCase):
         # There is no equality between items
         self.assertEqual(len(data), len(set(data)))
         # At worst a float32
-        self.assertIn(data.dtype.char, ['d', 'f'])
+        self.assertIn(data.dtype.kind, ['d', 'f'])
         self.assertLessEqual(data.dtype.itemsize, 32 / 8)
 
     def test_float_64(self):
@@ -248,7 +248,7 @@ class TestFabioH5(unittest.TestCase):
         # There is no equality between items
         self.assertEqual(len(data), len(set(data)))
         # At least a float64
-        self.assertIn(data.dtype.char, ['d', 'f'])
+        self.assertIn(data.dtype.kind, ['d', 'f'])
         self.assertGreaterEqual(data.dtype.itemsize, 64 / 8)
 
     def test_ub_matrix(self):
@@ -269,14 +269,14 @@ class TestFabioH5(unittest.TestCase):
         expected = numpy.array([4.08, 4.08, 4.08])
         self.assertIsNotNone(d)
         self.assertEquals(d.shape, (3, ))
-        self.assertIn(d.dtype.char, ['d', 'f'])
+        self.assertIn(d.dtype.kind, ['d', 'f'])
         numpy.testing.assert_array_almost_equal(d[...], expected)
 
         d = sample['unit_cell_alphabetagamma']
         expected = numpy.array([90.0, 90.0, 90.0])
         self.assertIsNotNone(d)
         self.assertEquals(d.shape, (3, ))
-        self.assertIn(d.dtype.char, ['d', 'f'])
+        self.assertIn(d.dtype.kind, ['d', 'f'])
         numpy.testing.assert_array_almost_equal(d[...], expected)
 
         d = sample['ub_matrix']
@@ -285,7 +285,7 @@ class TestFabioH5(unittest.TestCase):
                                  [1.08894, 1.08894, 9.28619e-17]]])
         self.assertIsNotNone(d)
         self.assertEquals(d.shape, (1, 3, 3))
-        self.assertIn(d.dtype.char, ['d', 'f'])
+        self.assertIn(d.dtype.kind, ['d', 'f'])
         numpy.testing.assert_array_almost_equal(d[...], expected)
 
     def test_interpretation_mca_edf(self):
@@ -510,6 +510,13 @@ class TestFabioH5WithEdf(unittest.TestCase):
         self.assertNotIn("/scan_0/instrument/detector_0/others/HeaderID", self.h5_image)
 
 
+if fabio is not None:
+    class _TestableFrameData(fabioh5.FrameData):
+        """Allow to test if the full data is reached."""
+        def _create_data(self):
+            raise RuntimeError("Not supposed to be called")
+
+
 class TestFabioH5WithFileSeries(unittest.TestCase):
 
     @classmethod
@@ -566,6 +573,13 @@ class TestFabioH5WithFileSeries(unittest.TestCase):
         h5_image = fabioh5.File(file_series=file_series)
         self._testH5Image(h5_image)
 
+    def testFrameDataCache(self):
+        file_series = fabioh5._FileSeries(self.edf_filenames)
+        reader = fabioh5.FabioReader(file_series=file_series)
+        frameData = _TestableFrameData("foo", reader)
+        self.assertEqual(frameData.dtype.kind, "i")
+        self.assertEqual(frameData.shape, (10, 3, 2))
+
 
 def suite():
     loadTests = unittest.defaultTestLoader.loadTestsFromTestCase
